@@ -32,7 +32,7 @@ from .pixelfunc import (ma, mask_good, mask_bad,
                        nside2order, order2nside,
                        isnsideok, isnpixok,
                        ring2nest, nest2ring, reorder,
-                       get_neighbours, get_all_neighbours, max_pixrad, get_interp_val,
+                       get_all_neighbours, max_pixrad, get_interp_val,
                        get_interp_weights,
                        fit_dipole, fit_monopole,
                        remove_dipole, remove_monopole,
