@@ -89,11 +89,9 @@ class TestSphtFunc(unittest.TestCase):
         smoothed_f90 = hp.ma(hp.read_map(os.path.join(self.path, 'data',
                   'wmap_band_iqumap_r9_7yr_W_v4_udgraded32_masked_smoothed10deg_fortran.fits'), (0,1,2)))
         # fortran does not restore the mask
-        for mm in smoothed_f90:
-            mm.mask = smoothed[0].mask
-        for i in range(3):
-            np.testing.assert_array_almost_equal(smoothed[i].filled(), smoothed_f90[i].filled(), decimal=6)
-        
+        smoothed_f90.mask = smoothed.mask
+        np.testing.assert_array_almost_equal(smoothed.filled(), smoothed_f90.filled(), decimal=6)
+
     def test_gauss_beam(self):
         idl_gauss_beam = np.array(pf.open(os.path.join(self.path, 'data', 'gaussbeam_10arcmin_lmax512_pol.fits'))[0].data).T
         gauss_beam = hp.gauss_beam(np.radians(10./60.), lmax=512, pol=True)
@@ -110,10 +108,11 @@ class TestSphtFunc(unittest.TestCase):
         tmp = np.empty(orig.size * 2)
         tmp[::2] = orig
         maps = [orig, orig.astype(np.float32), tmp[::2]]
-        for input in maps:
-            alm = hp.map2alm(input, iter=10)
-            output = hp.alm2map(alm, nside)
-            np.testing.assert_allclose(input, output, atol=1e-4)
+        for use_weights in [False, True]:
+            for input in maps:
+                alm = hp.map2alm(input, iter=10, use_weights=use_weights)
+                output = hp.alm2map(alm, nside)
+                np.testing.assert_allclose(input, output, atol=1e-4)
 
     def test_map2alm_pol(self):
         tmp = [np.empty(o.size*2) for o in self.mapiqu]
@@ -121,11 +120,34 @@ class TestSphtFunc(unittest.TestCase):
             t[::2] = o
         maps = [self.mapiqu, [o.astype(np.float32) for o in self.mapiqu],
                 [t[::2] for t in tmp]]
-        for input in maps:
-            alm = hp.map2alm(input, iter=10)
-            output = hp.alm2map(alm, 32)
-            for i, o in zip(input, output):
-                np.testing.assert_allclose(i, o, atol=1e-4)
+        for use_weights in [False, True]:
+            for input in maps:
+                alm = hp.map2alm(input, iter=10, use_weights=use_weights)
+                output = hp.alm2map(alm, 32)
+                for i, o in zip(input, output):
+                    np.testing.assert_allclose(i, o, atol=1e-4)
+
+    def test_map2alm_pol_gal_cut(self):
+        tmp = [np.empty(o.size*2) for o in self.mapiqu]
+        for t, o in zip(tmp, self.mapiqu):
+            t[::2] = o
+        maps = [self.mapiqu, [o.astype(np.float32) for o in self.mapiqu],
+                [t[::2] for t in tmp]]
+        for use_weights in [False, True]:
+            for input in maps:
+                gal_cut = 30
+                nside = hp.get_nside(input)
+                npix = hp.nside2npix(nside)
+                gal_mask = np.abs(hp.pix2ang(
+                    nside, np.arange(npix), lonlat=True)[1]) < gal_cut
+                alm = hp.map2alm(
+                    input, iter=10, use_weights=use_weights, gal_cut=gal_cut)
+                output = hp.alm2map(alm, 32)
+                for i, o in zip(input, output):
+                    # Testing requires low tolerances because of the
+                    # mask boundary
+                    i[gal_mask] = 0
+                    np.testing.assert_allclose(i, o, atol=1e-2)
 
     def test_rotate_alm(self):
         almigc = hp.map2alm(self.mapiqu)
@@ -137,6 +159,59 @@ class TestSphtFunc(unittest.TestCase):
             # FIXME: rtol=1e-6 works here, except on Debian with Python 3.4.
             np.testing.assert_allclose(i, o, rtol=1e-5)
 
+    def test_rotate_alm2(self):
+        # Test rotate_alm against the Fortran library
+        lmax = 64
+        nalm = hp.Alm.getsize(lmax)
+        alm = np.zeros([3, nalm], dtype=np.complex)
+        for i in range(3):
+            for ell in range(lmax+1):
+                for m in range(ell):
+                    ind = hp.Alm.getidx(lmax, ell, m)
+                    alm[i, ind] = (i+1)*10 + ell + 1j*m
+        psi = np.pi/3.
+        theta = 0.5
+        phi = 0.01
+        hp.rotate_alm(alm, psi, theta, phi)
+
+        ref_0_0_0 =  0.00000000000 + 0.00000000000j
+        ref_0_21_0 = -64.0056622444 + 0.00000000000j
+        ref_0_21_21 = -3.19617408364 + 2.00219590117j
+        ref_0_42_0 = 87.8201360825 + 0.00000000000j
+        ref_0_42_21 = -6.57242309702 + 50.1128079361j
+        ref_0_42_42 = 0.792592362074 - .928452597766j
+        ref_0_63_0 = -49.6732554742 + 0.00000000000j
+        ref_0_63_21 = -51.2812623888 - 61.6289129316j
+        ref_0_63_42 = -9.32823219430 + 79.0787993482j
+        ref_0_63_63 = -.157204566965 + 0.324692958700j
+        ref_1_0_0 = 0.00000000000 + 0.00000000000j
+        ref_1_21_0 = -85.5520809077 + 0.00000000000j
+        ref_1_21_21 = -3.57384285749 + 2.93255811219j
+        ref_1_42_0 = 107.541172254 + 0.00000000000j
+        ref_1_42_21 = -2.77944941833 + 57.1015322415j
+        ref_1_42_42 = 0.794212854046 - 1.10982745343j
+        ref_1_63_0 = -60.7153303746 + 0.00000000000j
+        ref_1_63_21 = -61.0915123767 - 65.9943878923j
+        ref_1_63_42 = -4.86354653261 + 86.5277253196j
+        ref_1_63_63 = -.147165377786 + 0.360474777237j
+
+        ref = np.array([
+            ref_0_0_0, ref_0_21_0, ref_0_21_21, ref_0_42_0, ref_0_42_21,
+            ref_0_42_42, ref_0_63_0, ref_0_63_21, ref_0_63_42, ref_0_63_63,
+            ref_1_0_0, ref_1_21_0, ref_1_21_21, ref_1_42_0, ref_1_42_21,
+            ref_1_42_42, ref_1_63_0, ref_1_63_21, ref_1_63_42, ref_1_63_63])
+
+        mine = []
+        for i in [0, 1]:
+            for ell in range(0, lmax+1, 21):
+                for m in range(0, ell+1, 21):
+                    ind = hp.Alm.getidx(lmax, ell, m)
+                    mine.append(alm[i, ind])
+
+        mine = np.array(ref)
+
+        np.testing.assert_allclose(ref, mine, rtol=1e-10)
+
     def test_accept_ma_allows_only_keywords(self):
         """ Test whether 'smoothing' wrapped with accept_ma works with only
             keyword arguments. """
