@@ -8,13 +8,16 @@ from ..visufunc import *
 from ..zoomtool import mollzoom
 
 class TestNoCrash(unittest.TestCase):
-    
+
     def setUp(self):
-        self.nside = 1
+        self.nside = 16
         self.m = np.arange(hp.nside2npix(self.nside), dtype=np.double)
         self.ma = self.m.copy()
         self.ma[3] = hp.UNSEEN
         self.ma = hp.ma(self.ma)
+        self.m2 = self.m.copy()
+        self.m2[100:] = hp.UNSEEN
+        self.ma2 = hp.ma(self.m2)
 
     def test_cartview_nocrash(self):
         cartview(self.m)
@@ -34,6 +37,9 @@ class TestNoCrash(unittest.TestCase):
     def test_mollzoom_nocrash(self):
         mollzoom(self.m)
 
+    def test_mollzoom_histnocrash(self):
+        mollzoom(self.m, norm='hist')
+
     def test_cartview_ma_nocrash(self):
         cartview(self.ma)
 
@@ -49,5 +55,8 @@ class TestNoCrash(unittest.TestCase):
     def test_mollzoom_ma_nocrash(self):
         mollzoom(self.ma)
 
+    def test_mollzoom_ma_hist_nocrash(self):
+        mollzoom(self.ma2, norm='hist')
+
     def test_azeqview_ma_nocrash(self):
         azeqview(self.ma)
